@@ -3,16 +3,18 @@ Like the old south.modelsparser, but using introspection where possible
 rather than direct inspection of models.py.
 """
 
+from __future__ import print_function
+
 import datetime
 import re
 import decimal
 
 from south.utils import get_attribute, auto_through
+from south.utils.py3 import text_type
 
 from django.db import models
 from django.db.models.base import ModelBase, Model
 from django.db.models.fields import NOT_PROVIDED
-from django.db.models import CASCADE, PROTECT, SET, SET_NULL, SET_DEFAULT, DO_NOTHING
 from django.conf import settings
 from django.utils.functional import Promise
 from django.contrib.contenttypes import generic
@@ -26,10 +28,13 @@ try:
 except ImportError:
     timezone = False
 
+
+# Define any converter functions first to prevent NameErrors
+
 def convert_on_delete_handler(value):
-    django_db_models_module = 'models' # relative to standard import 'django.db'
-    if django_db_models_module:
-        if value in (CASCADE, PROTECT, DO_NOTHING, SET_DEFAULT):
+    django_db_models_module = 'models'  # relative to standard import 'django.db'
+    if hasattr(models, "PROTECT"):
+        if value in (models.CASCADE, models.PROTECT, models.DO_NOTHING, models.SET_DEFAULT):
             # straightforward functions
             return '%s.%s' % (django_db_models_module, value.__name__)
         else:
@@ -37,28 +42,18 @@ def convert_on_delete_handler(value):
             func_name = getattr(value, '__name__', None)
             if func_name == 'set_on_delete':
                 # we must inspect the function closure to see what parameters were passed in
-                closure_contents = value.func_closure[0].cell_contents
+                closure_contents = value.__closure__[0].cell_contents
                 if closure_contents is None:
                     return "%s.SET_NULL" % (django_db_models_module)
                 # simple function we can perhaps cope with:
                 elif hasattr(closure_contents, '__call__'):
-                    # module_name = getattr(closure_contents, '__module__', None)
-                    # inner_func_name = getattr(closure_contents, '__name__', None)
-                    # if inner_func_name:
-                        # TODO there is no way of checking that module_name matches the
-                        # model file, which is the only code that will be imported in
-                        # the Fake ORM. Any other functions won't be available.
-                        # TODO this doesn't work anyway yet as even the app's models
-                        # file is not imported, contrary to the coments in
-                        # orm.LazyFakeORM.eval_in_context, which implies that
-                        # migrations are expected to import that.
-                        # return "%s.SET(%s)" % (django_db_models_module, inner_func_name)
-                    raise ValueError("Function for on_delete could not be serialized.")
+                    raise ValueError("South does not support on_delete with SET(function) as values.")
                 else:
-                    # an actual value rather than a sentinel function - insanity
-                    raise ValueError("on_delete=SET with a model instance is not supported.")
-                    
-    raise ValueError("%s was not recognized as a valid model deletion handler. Possible values: %s." % (value, ', '.join(f.__name__ for f in (CASCADE, PROTECT, SET, SET_NULL, SET_DEFAULT, DO_NOTHING))))
+                    # Attempt to serialise the value
+                    return "%s.SET(%s)" % (django_db_models_module, value_clean(closure_contents))
+        raise ValueError("%s was not recognized as a valid model deletion handler. Possible values: %s." % (value, ', '.join(f.__name__ for f in (models.CASCADE, models.PROTECT, models.SET, models.SET_NULL, models.SET_DEFAULT, models.DO_NOTHING))))
+    else:
+        raise ValueError("on_delete argument encountered in Django version that does not support it")
 
 # Gives information about how to introspect certain fields.
 # This is a list of triples; the first item is a list of fields it applies to,
@@ -71,6 +66,7 @@ def convert_on_delete_handler(value):
 # is an optional dict.
 #
 # The introspector uses the combination of all matching entries, in order.
+                                     
 introspection_details = [
     (
         (models.Field, ),
@@ -90,13 +86,13 @@ introspection_details = [
     (
         (models.ForeignKey, models.OneToOneField),
         [],
-        {
-            "to": ["rel.to", {}],
-            "to_field": ["rel.field_name", {"default_attr": "rel.to._meta.pk.name"}],
-            "related_name": ["rel.related_name", {"default": None}],
-            "db_index": ["db_index", {"default": True}],
-            "on_delete": ["rel.on_delete", {"default": CASCADE, "is_django_function": True, "converter": convert_on_delete_handler, }],
-        },
+        dict([
+            ("to", ["rel.to", {}]),
+            ("to_field", ["rel.field_name", {"default_attr": "rel.to._meta.pk.name"}]),
+            ("related_name", ["rel.related_name", {"default": None}]),
+            ("db_index", ["db_index", {"default": True}]),
+            ("on_delete", ["rel.on_delete", {"default": getattr(models, "CASCADE", None), "is_django_function": True, "converter": convert_on_delete_handler, "ignore_missing": True}])
+        ])
     ),
     (
         (models.ManyToManyField,),
@@ -181,13 +177,11 @@ meta_details = {
     "db_table": ["db_table", {"default_attr_concat": ["%s_%s", "app_label", "module_name"]}],
     "db_tablespace": ["db_tablespace", {"default": settings.DEFAULT_TABLESPACE}],
     "unique_together": ["unique_together", {"default": []}],
+    "index_together": ["index_together", {"default": []}],
     "ordering": ["ordering", {"default": []}],
     "proxy": ["proxy", {"default": False, "ignore_missing": True}],
 }
 
-# 2.4 compatability
-any = lambda x: reduce(lambda y, z: y or z, x, False)
-
 
 def add_introspection_rules(rules=[], patterns=[]):
     "Allows you to add some introspection rules at runtime, e.g. for 3rd party apps."
@@ -196,11 +190,13 @@ def add_introspection_rules(rules=[], patterns=[]):
     allowed_fields.extend(patterns)
     introspection_details.extend(rules)
 
+
 def add_ignored_fields(patterns):
     "Allows you to add some ignore field patterns."
     assert isinstance(patterns, (list, tuple))
     ignored_fields.extend(patterns)
     
+
 def can_ignore(field):
     """
     Returns True if we know for certain that we can ignore this field, False
@@ -212,6 +208,7 @@ def can_ignore(field):
             return True
     return False
 
+
 def can_introspect(field):
     """
     Returns True if we are allowed to introspect this field, False otherwise.
@@ -267,7 +264,7 @@ def get_value(field, descriptor):
             
     # Lazy-eval functions get eval'd.
     if isinstance(value, Promise):
-        value = unicode(value)
+        value = text_type(value)
     # If the value is the same as the default, omit it for clarity
     if "default" in options and value == options['default']:
         raise IsDefault
@@ -298,7 +295,7 @@ def value_clean(value, options={}):
     "Takes a value and cleans it up (so e.g. it has timezone working right)"
     # Lazy-eval functions get eval'd.
     if isinstance(value, Promise):
-        value = unicode(value)
+        value = text_type(value)
     # Callables get called.
     if not options.get('is_django_function', False) and callable(value) and not isinstance(value, ModelBase):
         # Datetime.datetime.now is special, as we can access it from the eval
@@ -356,6 +353,7 @@ def value_clean(value, options={}):
     else:
         return repr(value)
 
+
 def introspector(field):
     """
     Given a field, introspects its definition triple.
@@ -404,7 +402,7 @@ def get_model_fields(model, m2m=False):
         # Does it define a south_field_triple method?
         if hasattr(field, "south_field_triple"):
             if NOISY:
-                print " ( Nativing field: %s" % field.name
+                print(" ( Nativing field: %s" % field.name)
             field_defs[field.name] = field.south_field_triple()
         # Can we introspect it?
         elif can_introspect(field):
@@ -420,7 +418,7 @@ def get_model_fields(model, m2m=False):
         # Shucks, no definition!
         else:
             if NOISY:
-                print " ( Nodefing field: %s" % field.name
+                print(" ( Nodefing field: %s" % field.name)
             field_defs[field.name] = None
     
     # If they've used the horrific hack that is order_with_respect_to, deal with
